@@ -31,6 +31,8 @@
 #define BUFLEN 2048
 #define SEND_BUF_SIZE 1024
 #define MAX_ARGS 512
+#define IDLE_TIMEOUT 30
+#define RANGE_SIZE 128
 
 extern db_param_t _db_params;
 #define DBPFX _db_params.pfx
@@ -177,8 +179,6 @@ struct ImapSession * dbmail_imap_session_new(void)
 {
 	struct ImapSession * self;
 
-	g_mime_init(0);
-
 	/* init: cache */
 	if (init_cache() != 0)
 		return NULL;
@@ -186,6 +186,7 @@ struct ImapSession * dbmail_imap_session_new(void)
 	self = g_new0(struct ImapSession,1);
 
 	self->args = g_new0(char *, MAX_ARGS);
+	self->buff = g_string_new("");
 
 	dbmail_imap_session_resetFi(self);
 	
@@ -200,7 +201,8 @@ struct ImapSession * dbmail_imap_session_resetFi(struct ImapSession * self)
 	}
 	
 	dbmail_imap_session_bodyfetch_free(self);
-	memset(self->fi,'\0',sizeof(fetch_items_t));
+	g_free(self->fi);
+	self->fi = g_new0(fetch_items_t,1);
 	return self;
 }
      
@@ -276,8 +278,8 @@ void dbmail_imap_session_delete(struct ImapSession * self)
 
 	dbmail_imap_session_fetch_free(self);
 	dbmail_imap_session_args_free(self, TRUE);
-
-	g_mime_shutdown();
+	
+	g_string_free(self->buff,TRUE);
 
 	g_free(self);
 }
@@ -699,6 +701,7 @@ int dbmail_imap_session_fetch_parse_args(struct ImapSession * self)
 	return 1; //theres more...
 }
 
+
 GTree * dbmail_imap_session_get_msginfo(struct ImapSession *self, GTree *ids)
 {
 
@@ -710,8 +713,9 @@ GTree * dbmail_imap_session_get_msginfo(struct ImapSession *self, GTree *ids)
 	GList *l, *t;
 	u64_t *uid, *lo, *hi;
 	u64_t id;
-	char query[DEF_QUERYSIZE];
+	char query[DEF_QUERYSIZE], range[RANGE_SIZE];
 	memset(query,0,DEF_QUERYSIZE);
+	memset(range,0,RANGE_SIZE);
 	
 	if (! (ids && g_tree_nnodes(ids)>0))
 		return NULL;
@@ -730,20 +734,24 @@ GTree * dbmail_imap_session_get_msginfo(struct ImapSession *self, GTree *ids)
 
 	k = 0;
 	to_char_str = date2char_str("internal_date");
-
 		
 	db_free_result();
 
+	if (*lo == *hi) 
+		snprintf(range,RANGE_SIZE,"= %llu", *lo);
+	else
+		snprintf(range,RANGE_SIZE,"BETWEEN %llu AND %llu", *lo, *hi);
+
 	snprintf(query, DEF_QUERYSIZE,
 		 "SELECT seen_flag, answered_flag, deleted_flag, flagged_flag, "
 		 "draft_flag, recent_flag, %s, rfcsize, message_idnr "
 		 "FROM %smessages msg, %sphysmessage pm "
 		 "WHERE pm.id = msg.physmessage_id "
-		 "AND message_idnr BETWEEN %llu AND %llu "
-		 "AND mailbox_idnr = %llu AND status IN (%d,%d) "
+		 "AND message_idnr %s "
+		 "AND mailbox_idnr = %llu AND status IN (%d,%d,%d) "
 		 "ORDER BY message_idnr ASC",to_char_str,DBPFX,DBPFX,
-		 *lo, *hi, ud->mailbox.uid,
-		 MESSAGE_STATUS_NEW, MESSAGE_STATUS_SEEN);
+		 range, ud->mailbox.uid,
+		 MESSAGE_STATUS_NEW, MESSAGE_STATUS_SEEN,MESSAGE_STATUS_DELETE);
 	g_free(to_char_str);
 
 	if (db_query(query) == -1) {
@@ -752,6 +760,7 @@ GTree * dbmail_imap_session_get_msginfo(struct ImapSession *self, GTree *ids)
 	}
 
 	if ((nrows = db_num_rows()) == 0) {
+		TRACE(TRACE_ERROR, "empty result set");
 		db_free_result();
 		return NULL;
 	}
@@ -789,7 +798,6 @@ GTree * dbmail_imap_session_get_msginfo(struct ImapSession *self, GTree *ids)
 		g_tree_insert(msginfo, uid, result); 
 	}
 
-	
 	db_free_result();
 
 	return msginfo;
@@ -798,8 +806,7 @@ GTree * dbmail_imap_session_get_msginfo(struct ImapSession *self, GTree *ids)
 #define SEND_SPACE if (self->fi->isfirstfetchout) \
 				self->fi->isfirstfetchout = 0; \
 			else \
-				dbmail_imap_session_printf(self, " ")
-
+				dbmail_imap_session_buff_append(self, " ")
 
 static int _fetch_get_items(struct ImapSession *self, u64_t *uid)
 {
@@ -822,22 +829,22 @@ static int _fetch_get_items(struct ImapSession *self, u64_t *uid)
 
 	if (self->fi->getInternalDate) {
 		SEND_SPACE;
-		dbmail_imap_session_printf(self, "INTERNALDATE \"%s\"", date_sql2imap(msginfo->internaldate));
+		dbmail_imap_session_buff_append(self, "INTERNALDATE \"%s\"", date_sql2imap(msginfo->internaldate));
 	}
 	if (self->fi->getSize) {
 		SEND_SPACE;
-		dbmail_imap_session_printf(self, "RFC822.SIZE %llu", msginfo->rfcsize);
+		dbmail_imap_session_buff_append(self, "RFC822.SIZE %llu", msginfo->rfcsize);
 	}
 	if (self->fi->getFlags) {
 		SEND_SPACE;
 		s = imap_flags_as_string(msginfo);
-		dbmail_imap_session_printf(self,"FLAGS %s",s);
+		dbmail_imap_session_buff_append(self,"FLAGS %s",s);
 		g_free(s);
 
 	}
 	if (self->fi->getUID) {
 		SEND_SPACE;
-		dbmail_imap_session_printf(self, "UID %llu", msginfo->uid);
+		dbmail_imap_session_buff_append(self, "UID %llu", msginfo->uid);
 	}
 
 	if (self->fi->getMIME_IMB) {
@@ -849,7 +856,7 @@ static int _fetch_get_items(struct ImapSession *self, u64_t *uid)
 			dbmail_imap_session_printf(self, "\r\n* BYE error fetching body structure\r\n");
 			return -1;
 		}
-		dbmail_imap_session_printf(self, "BODYSTRUCTURE %s", s);
+		dbmail_imap_session_buff_append(self, "BODYSTRUCTURE %s", s);
 		g_free(s);
 	}
 
@@ -862,7 +869,7 @@ static int _fetch_get_items(struct ImapSession *self, u64_t *uid)
 			dbmail_imap_session_printf(self, "\r\n* BYE error fetching body\r\n");
 			return -1;
 		}
-		dbmail_imap_session_printf(self, "BODY %s",s);
+		dbmail_imap_session_buff_append(self, "BODY %s",s);
 		g_free(s);
 	}
 
@@ -878,7 +885,8 @@ static int _fetch_get_items(struct ImapSession *self, u64_t *uid)
 		SEND_SPACE;
 
 		_imap_cache_update(self, DBMAIL_MESSAGE_FILTER_FULL);
-		dbmail_imap_session_printf(self, "RFC822 {%llu}\r\n", cached_msg.dumpsize);
+		dbmail_imap_session_buff_append(self, "RFC822 {%llu}\r\n", cached_msg.dumpsize);
+		dbmail_imap_session_buff_flush(self);
 		send_data(self->ci->tx, cached_msg.memdump, cached_msg.dumpsize);
 
 		if (self->fi->getRFC822)
@@ -892,7 +900,8 @@ static int _fetch_get_items(struct ImapSession *self, u64_t *uid)
 		
 		_imap_cache_update(self, DBMAIL_MESSAGE_FILTER_FULL);
 		if (dbmail_imap_session_bodyfetch_get_last_octetcnt(self) == 0) {
-			dbmail_imap_session_printf(self, "BODY[] {%llu}\r\n", cached_msg.dumpsize);
+			dbmail_imap_session_buff_append(self, "BODY[] {%llu}\r\n", cached_msg.dumpsize);
+			dbmail_imap_session_buff_flush(self);
 			send_data(self->ci->tx, cached_msg.memdump, cached_msg.dumpsize);
 		} else {
 			mseek(cached_msg.memdump, dbmail_imap_session_bodyfetch_get_last_octetstart(self), SEEK_SET);
@@ -901,8 +910,9 @@ static int _fetch_get_items(struct ImapSession *self, u64_t *uid)
 			    ? (((long long)cached_msg.dumpsize) - dbmail_imap_session_bodyfetch_get_last_octetstart(self)) 
 			    : dbmail_imap_session_bodyfetch_get_last_octetcnt(self);
 
-			dbmail_imap_session_printf(self, "BODY[]<%llu> {%llu}\r\n", 
+			dbmail_imap_session_buff_append(self, "BODY[]<%llu> {%llu}\r\n", 
 					dbmail_imap_session_bodyfetch_get_last_octetstart(self), actual_cnt);
+			dbmail_imap_session_buff_flush(self);
 			send_data(self->ci->tx, cached_msg.memdump, actual_cnt);
 		}
 
@@ -916,7 +926,8 @@ static int _fetch_get_items(struct ImapSession *self, u64_t *uid)
 		SEND_SPACE;
 
 		tmpdumpsize = _imap_cache_update(self,DBMAIL_MESSAGE_FILTER_HEAD);
-		dbmail_imap_session_printf(self, "RFC822.HEADER {%llu}\r\n", tmpdumpsize);
+		dbmail_imap_session_buff_append(self, "RFC822.HEADER {%llu}\r\n", tmpdumpsize);
+		dbmail_imap_session_buff_flush(self);
 		send_data(self->ci->tx, cached_msg.tmpdump, tmpdumpsize);
 	}
 
@@ -925,7 +936,8 @@ static int _fetch_get_items(struct ImapSession *self, u64_t *uid)
 		SEND_SPACE;
 
 		tmpdumpsize = _imap_cache_update(self,DBMAIL_MESSAGE_FILTER_BODY);
-		dbmail_imap_session_printf(self, "RFC822.TEXT {%llu}\r\n", tmpdumpsize);
+		dbmail_imap_session_buff_append(self, "RFC822.TEXT {%llu}\r\n", tmpdumpsize);
+		dbmail_imap_session_buff_flush(self);
 		send_data(self->ci->tx, cached_msg.tmpdump, tmpdumpsize);
 
 		self->fi->setseen = 1;
@@ -943,23 +955,23 @@ static int _fetch_get_items(struct ImapSession *self, u64_t *uid)
 		   flag be set! */
 		result = acl_has_right(&ud->mailbox, ud->userid, ACL_RIGHT_SEEN);
 		if (result == -1) {
-			dbmail_imap_session_printf(self, "\r\n *BYE internal dbase error\r\n");
+			dbmail_imap_session_buff_append(self, "\r\n *BYE internal dbase error\r\n");
 			return -1;
 		}
 		
 		if (result == 1) {
 			result = db_set_msgflag(self->msg_idnr, ud->mailbox.uid, setSeenSet, IMAPFA_ADD);
 			if (result == -1) {
-				dbmail_imap_session_printf(self, "\r\n* BYE internal dbase error\r\n");
+				dbmail_imap_session_buff_append(self, "\r\n* BYE internal dbase error\r\n");
 				return -1;
 			}
 		}
 
 		self->fi->getFlags = 1;
-		dbmail_imap_session_printf(self, " ");
+		dbmail_imap_session_buff_append(self, " ");
 	}
 
-	dbmail_imap_session_printf(self, ")\r\n");
+	dbmail_imap_session_buff_append(self, ")\r\n");
 
 	return 0;
 }
@@ -973,12 +985,18 @@ static gboolean _do_fetch(u64_t *uid, gpointer UNUSED value, struct ImapSession
 
 	g_return_val_if_fail(id,TRUE);
 	
-	dbmail_imap_session_printf(self, "* %llu FETCH (", *id);
+	dbmail_imap_session_buff_append(self, "* %llu FETCH (", *id);
 
 	/* go fetch the items */
 	fflush(self->ci->tx);
-	if (_fetch_get_items(self,uid) < 0)
+	if (_fetch_get_items(self,uid) < 0) {
+		self->error = TRUE;
 		return TRUE;
+	}
+
+	if (self->buff->len > 4096)
+		dbmail_imap_session_buff_flush(self);
+
 	return FALSE;
 }
 
@@ -997,7 +1015,11 @@ int dbmail_imap_session_fetch_get_items(struct ImapSession *self)
 	if (! self->ids)
 		TRACE(TRACE_INFO, "self->ids is NULL");
 	else {
+		dbmail_imap_session_buff_clear(self);
 		g_tree_foreach(self->ids, (GTraverseFunc) _do_fetch, self);
+		dbmail_imap_session_buff_flush(self);
+		if (self->error)
+			return -1;
 		dbmail_imap_session_mailbox_update_recent(self);
 	}
 	return 0;
@@ -1016,13 +1038,14 @@ static void imap_cache_send_tmpdump(struct ImapSession *self, body_fetch_t *body
 
 	if (bodyfetch->octetcnt > 0) {
 		cnt = get_dumpsize(bodyfetch, size);
-		dbmail_imap_session_printf(self, "]<%llu> {%llu}\r\n", bodyfetch->octetstart, cnt);
+		dbmail_imap_session_buff_append(self, "]<%llu> {%llu}\r\n", bodyfetch->octetstart, cnt);
 		mseek(cached_msg.tmpdump, bodyfetch->octetstart, SEEK_SET);
 	} else {
 		cnt = size;
-		dbmail_imap_session_printf(self, "] {%llu}\r\n", size);
+		dbmail_imap_session_buff_append(self, "] {%llu}\r\n", size);
 		mrewind(cached_msg.tmpdump);
 	}
+	dbmail_imap_session_buff_flush(self);
 	send_data(self->ci->tx, cached_msg.tmpdump, cnt);
 }
 
@@ -1032,13 +1055,15 @@ static void imap_cache_send_tmpdump(struct ImapSession *self, body_fetch_t *body
 static void _fetch_envelopes(struct ImapSession *self)
 {
 	unsigned i=0, rows=0;
-	GString *q = g_string_new("");
+	GString *q;
 	gchar *env, *s;
 	u64_t *mid;
 	u64_t id;
 	static int lo = 0;
 	static u64_t hi = 0;
+	char range[RANGE_SIZE];
 	GList *last;
+	memset(range,0,RANGE_SIZE);
 
 	if (! self->envelopes) {
 		self->envelopes = g_tree_new_full((GCompareDataFunc)ucmp,NULL,(GDestroyNotify)g_free,(GDestroyNotify)g_free);
@@ -1047,7 +1072,7 @@ static void _fetch_envelopes(struct ImapSession *self)
 	}
 
 	if ((s = g_tree_lookup(self->envelopes, &(self->msg_idnr))) != NULL) {
-		dbmail_imap_session_printf(self, "ENVELOPE %s", s?s:"");
+		dbmail_imap_session_buff_append(self, "ENVELOPE %s", s?s:"");
 		return;
 	}
 
@@ -1057,16 +1082,24 @@ static void _fetch_envelopes(struct ImapSession *self)
 		last = g_list_last(self->ids_list);
 	hi = *(u64_t *)last->data;
 
+	if (self->msg_idnr == hi)
+		snprintf(range,RANGE_SIZE,"= %llu", self->msg_idnr);
+	else
+		snprintf(range,RANGE_SIZE,"BETWEEN %llu AND %llu", self->msg_idnr, hi);
+
+        q = g_string_new("");
 	g_string_printf(q,"SELECT message_idnr,envelope "
 			"FROM %senvelope e "
 			"JOIN %smessages m ON m.physmessage_id=e.physmessage_id "
 			"WHERE m.mailbox_idnr = %llu "
-			"AND message_idnr BETWEEN %llu AND %llu ",
+			"AND message_idnr %s",
 			DBPFX, DBPFX,  
-			self->mailbox->id, self->msg_idnr, hi);
+			self->mailbox->id, range);
 	
-	if (db_query(q->str)==-1)
+	if (db_query(q->str)==-1) {
+                g_string_free(q,TRUE);
 		return;
+        }
 	
 	rows = db_num_rows();
 	
@@ -1091,7 +1124,7 @@ static void _fetch_envelopes(struct ImapSession *self)
 	lo += QUERY_BATCHSIZE;
 
 	s = g_tree_lookup(self->envelopes, &(self->msg_idnr));
-	dbmail_imap_session_printf(self, "ENVELOPE %s", s?s:"");
+	dbmail_imap_session_buff_append(self, "ENVELOPE %s", s?s:"");
 }
 
 void _send_headers(struct ImapSession *self, const body_fetch_t *bodyfetch, gboolean not, const gchar *s)
@@ -1100,10 +1133,10 @@ void _send_headers(struct ImapSession *self, const body_fetch_t *bodyfetch, gboo
 	gchar *tmp;
 	GString *ts;
 
-	dbmail_imap_session_printf(self,"HEADER.FIELDS%s %s] ", not ? ".NOT" : "", bodyfetch->hdrplist);
+	dbmail_imap_session_buff_append(self,"HEADER.FIELDS%s %s] ", not ? ".NOT" : "", bodyfetch->hdrplist);
 
 	if (!s) {
-		dbmail_imap_session_printf(self, "{2}\r\n\r\n");
+		dbmail_imap_session_buff_append(self, "{2}\r\n\r\n");
 		return;
 	}
 
@@ -1120,12 +1153,12 @@ void _send_headers(struct ImapSession *self, const body_fetch_t *bodyfetch, gboo
 		tmp = get_crlf_encoded(ts->str);
 		cnt = strlen(tmp);
 		
-		dbmail_imap_session_printf(self, "<%llu> {%llu}\r\n%s\r\n", 
+		dbmail_imap_session_buff_append(self, "<%llu> {%llu}\r\n%s\r\n", 
 				bodyfetch->octetstart, cnt+2, tmp);
 	} else {
 		tmp = get_crlf_encoded(ts->str);
 		cnt = strlen(tmp);
-		dbmail_imap_session_printf(self, "{%llu}\r\n%s\r\n", cnt+2, tmp);
+		dbmail_imap_session_buff_append(self, "{%llu}\r\n%s\r\n", cnt+2, tmp);
 	}
 
 	g_string_free(ts,TRUE);
@@ -1146,6 +1179,8 @@ static void _fetch_headers(struct ImapSession *self, body_fetch_t *bodyfetch, gb
 	static int lo = 0;
 	static u64_t hi = 0;
 	static u64_t ceiling = 0;
+	char range[RANGE_SIZE];
+	memset(range,0,RANGE_SIZE);
 
 	if (! self->headers) {
 		TRACE(TRACE_DEBUG, "init self->headers");
@@ -1191,16 +1226,20 @@ static void _fetch_headers(struct ImapSession *self, body_fetch_t *bodyfetch, gb
 		last = g_list_last(self->ids_list);
 	hi = *(u64_t *)last->data;
 
+	if (self->msg_idnr == hi)
+		snprintf(range,RANGE_SIZE,"= %llu", self->msg_idnr);
+	else
+		snprintf(range,RANGE_SIZE,"BETWEEN %llu AND %llu", self->msg_idnr, hi);
+
 	g_string_printf(q,"SELECT message_idnr,headername,headervalue "
 			"FROM %sheadervalue v "
 			"JOIN %smessages m ON v.physmessage_id=m.physmessage_id "
 			"JOIN %sheadername n ON v.headername_id=n.id "
 			"WHERE m.mailbox_idnr = %llu "
-			"AND message_idnr BETWEEN %llu AND %llu "
+			"AND message_idnr %s "
 			"AND lower(headername) %s IN ('%s')",
 			DBPFX, DBPFX, DBPFX,
-			self->mailbox->id,
-			self->msg_idnr, hi, 
+			self->mailbox->id, range, 
 			not?"NOT":"", bodyfetch->hdrnames);
 	
 	if (db_query(q->str)==-1)
@@ -1219,7 +1258,7 @@ static void _fetch_headers(struct ImapSession *self, body_fetch_t *bodyfetch, gb
 		*mid = id;
 		
 		fld = (char *)db_get_result(i,1);
-		val = convert_8bit_db_to_mime((char *)db_get_result(i,2));
+		val = dbmail_iconv_db_to_utf7((char *)db_get_result(i,2));
 		TRACE(TRACE_DEBUG,"got hdrval [%s]", val);
 		
 		old = g_tree_lookup(self->headers, (gconstpointer)mid);
@@ -1273,57 +1312,57 @@ static int _imap_show_body_section(body_fetch_t *bodyfetch, gpointer data)
 
 	if (! self->fi->noseen)
 		self->fi->setseen = 1;
-	dbmail_imap_session_printf(self, "BODY[%s", bodyfetch->partspec);
+	dbmail_imap_session_buff_append(self, "BODY[%s", bodyfetch->partspec);
 
 	switch (bodyfetch->itemtype) {
 
 	case BFIT_TEXT:
-		dbmail_imap_session_printf(self, "TEXT");
+		dbmail_imap_session_buff_append(self, "TEXT");
 		/* fall through */
 		
 	case BFIT_TEXT_SILENT:
 		if (!part)
-			dbmail_imap_session_printf(self, "] NIL");
+			dbmail_imap_session_buff_append(self, "] NIL");
 		else {
 			tmp = imap_get_logical_part(part,"TEXT");
 			tmpdumpsize = _imap_cache_set_dump(tmp,IMAP_CACHE_TMPDUMP);
 			g_free(tmp);
 			
 			if (!tmpdumpsize) 
-				dbmail_imap_session_printf(self, "] NIL");
+				dbmail_imap_session_buff_append(self, "] NIL");
 			else 
 				imap_cache_send_tmpdump(self,bodyfetch,tmpdumpsize);
 		}
 		break;
 
 	case BFIT_HEADER:
-		dbmail_imap_session_printf(self, "HEADER");
+		dbmail_imap_session_buff_append(self, "HEADER");
 		if (!part)
-			dbmail_imap_session_printf(self, "] NIL");
+			dbmail_imap_session_buff_append(self, "] NIL");
 		else {
 			tmp = imap_get_logical_part(part,"HEADER");
 			tmpdumpsize = _imap_cache_set_dump(tmp,IMAP_CACHE_TMPDUMP);
 			g_free(tmp);
 			
 			if (!tmpdumpsize) 
-				dbmail_imap_session_printf(self, "] NIL");
+				dbmail_imap_session_buff_append(self, "] NIL");
 			else 
 				imap_cache_send_tmpdump(self,bodyfetch,tmpdumpsize);
 		}
 		break;
 		
 	case BFIT_MIME:
-		dbmail_imap_session_printf(self, "MIME");
+		dbmail_imap_session_buff_append(self, "MIME");
 
 		if (!part)
-			dbmail_imap_session_printf(self, "NIL");
+			dbmail_imap_session_buff_append(self, "NIL");
 		else {
 			tmp = imap_get_logical_part(part,"MIME");
 			tmpdumpsize = _imap_cache_set_dump(tmp,IMAP_CACHE_TMPDUMP);
 			g_free(tmp);
 
 			if (!tmpdumpsize)
-				dbmail_imap_session_printf(self, "NIL");
+				dbmail_imap_session_buff_append(self, "NIL");
 			else
 				imap_cache_send_tmpdump(self,bodyfetch,tmpdumpsize);
 		}
@@ -1347,6 +1386,13 @@ static int _imap_show_body_section(body_fetch_t *bodyfetch, gpointer data)
 
 
 
+int client_is_authenticated(struct ImapSession * self)
+{
+	imap_userdata_t *ud = (imap_userdata_t *) self->ci->userData;
+
+	return (ud->state != IMAPCS_NON_AUTHENTICATED);
+}
+
 /*
  * check_state_and_args()
  *
@@ -1396,7 +1442,34 @@ int check_state_and_args(struct ImapSession * self, const char *command, int min
 	/* succes */
 	return 1;
 }
-	
+
+void dbmail_imap_session_buff_clear(struct ImapSession *self)
+{
+	g_string_printf(self->buff, "%s", "");
+}	
+
+void dbmail_imap_session_buff_append(struct ImapSession *self, char *message, ...)
+{
+	va_list ap;
+	gchar *ln;
+	assert(message);
+
+	va_start(ap, message);
+	ln = g_strdup_vprintf(message, ap);
+	va_end(ap);
+
+	g_string_append_printf(self->buff, "%s", ln);
+
+	g_free(ln);
+
+}
+
+void dbmail_imap_session_buff_flush(struct ImapSession *self)
+{
+	dbmail_imap_session_printf(self, "%s", self->buff->str);
+	dbmail_imap_session_buff_clear(self);
+}
+
 /* Returns -1 on error, -2 on serious error. */
 int dbmail_imap_session_printf(struct ImapSession * self, char * message, ...)
 {
@@ -1467,7 +1540,7 @@ int dbmail_imap_session_discard_to_eol(struct ImapSession *self)
 	/* loop until we get a newline terminated chunk */
 	while (!done)  {
 		memset(buffer, 0, MAX_LINESIZE);
-		alarm(ci->timeout);
+		alarm(self->timeout);
 		if (fgets(buffer, MAX_LINESIZE, ci->rx) == NULL) {
 			alarm(0);
 			TRACE(TRACE_ERROR, "error reading from client");
@@ -1499,7 +1572,7 @@ int dbmail_imap_session_readln(struct ImapSession *self, char * buffer)
 	assert(buffer);
 	memset(buffer, 0, MAX_LINESIZE);
 
-	alarm(ci->timeout);
+	alarm(self->timeout);
 	alarm_occured = 0;
 
 	fgets(buffer, MAX_LINESIZE, ci->rx);
@@ -1572,7 +1645,8 @@ int dbmail_imap_session_handle_auth(struct ImapSession * self, char * username,
 
 	/* update client info */
 	ud->userid = userid;
-	ud->state = IMAPCS_AUTHENTICATED;
+
+	dbmail_imap_session_set_state(self,IMAPCS_AUTHENTICATED);
 
 	return 0;
 
@@ -1701,7 +1775,7 @@ static gboolean imap_msginfo_notify(u64_t *uid, msginfo_t *msginfo, struct ImapS
 	// EXPUNGE
 	switch (self->command_type) {
 		case IMAP_COMM_NOOP:
-                        TRACE(TRACE_DEBUG,"send expunge?");
+		case IMAP_COMM_IDLE:
 			if (! newmsginfo) {
 				dbmail_imap_session_printf(self, "* %llu EXPUNGE\r\n", *msn);
 				dbmail_mailbox_remove_uid(self->mailbox, uid);
@@ -1747,14 +1821,18 @@ int dbmail_imap_session_mailbox_status(struct ImapSession * self, gboolean updat
 	*/
 
 	GTree *oldmsginfo, *msginfo = NULL;
-	unsigned oldexists, oldrecent;
+	unsigned oldexists, oldrecent, olduidnext;
         struct DbmailMailbox *mailbox = NULL;
+	gboolean showexists = FALSE;
 	imap_userdata_t *ud = (imap_userdata_t *) self->ci->userData;
-	if (! ud->state == IMAPCS_SELECTED)
+	if (ud->state != IMAPCS_SELECTED) {
+		TRACE(TRACE_DEBUG,"do nothing: state [%d]", ud->state);
 		return 0;
+	}
 
 	oldexists = ud->mailbox.exists;
 	oldrecent = ud->mailbox.recent;
+	olduidnext = ud->mailbox.msguidnext;
         oldmsginfo = self->msginfo;
 
 	if (update) {
@@ -1765,13 +1843,17 @@ int dbmail_imap_session_mailbox_status(struct ImapSession * self, gboolean updat
 		dbmail_mailbox_open(mailbox); 
 
                 // re-read flags and counters
-		if ((db_getmailbox(&(ud->mailbox))) == -1) 
+		if ((db_getmailbox(&(ud->mailbox))) == -1) {
+			dbmail_mailbox_free(mailbox);
 			return -1;
+		}
 
 		if ((msginfo = dbmail_imap_session_get_msginfo(self, mailbox->ids)) == NULL) {
 			TRACE(TRACE_DEBUG,"unable to retrieve msginfo");
-			if (ud->mailbox.exists > 0)
+			if (ud->mailbox.exists > 0) {
+				dbmail_mailbox_free(mailbox);
 				return -1;
+			}
 		}
 	}
 
@@ -1779,12 +1861,16 @@ int dbmail_imap_session_mailbox_status(struct ImapSession * self, gboolean updat
 	/* msg counts */
 	// EXPUNGE
 	switch (self->command_type) {
+		case IMAP_COMM_IDLE:
 		case IMAP_COMM_NOOP:
 		case IMAP_COMM_CHECK:
 		case IMAP_COMM_SELECT:
 		case IMAP_COMM_EXAMINE:
 	
-		if ((!update) || (ud->mailbox.exists >= oldexists)) // never decrements
+		if ((olduidnext != ud->mailbox.msguidnext) && (ud->mailbox.exists >= oldexists))
+			showexists = TRUE;
+
+		if ((!update) || (showexists)) // never decrements
 			dbmail_imap_session_printf(self, "* %u EXISTS\r\n", ud->mailbox.exists);
 
 		if ((!update) || (ud->mailbox.recent != oldrecent))
@@ -1915,6 +2001,68 @@ int dbmail_imap_session_mailbox_open(struct ImapSession * self, const char * mai
 	return 0;
 }
 
+#define IDLE_BUFFER 8
+int dbmail_imap_session_idle(struct ImapSession *self)
+{
+	char buffer[IDLE_BUFFER];
+	int result = 0, idle_timeout = IDLE_TIMEOUT;
+	clientinfo_t *ci = self->ci;
+	field_t val;
+	static int idle_keepalive = 0;
+
+	GETCONFIGVALUE("idle_timeout", "IMAP", val);
+	if ( strlen(val) && (idle_timeout = atoi(val)) <= 0 ) {
+		TRACE(TRACE_ERROR, "illegal value for idle_timeout [%s]", val);
+		idle_timeout = IDLE_TIMEOUT;	
+	}
+
+	memset(buffer, 0, IDLE_BUFFER);
+	dbmail_imap_session_mailbox_status(self,TRUE);
+	dbmail_imap_session_printf(self, "+ idling\r\n");
+	
+	while (1) {
+
+		if (ferror(ci->tx) || feof(ci->rx)) {
+			int serr = errno;
+			TRACE(TRACE_DEBUG,"error on client socket [%s]", strerror(serr));
+			errno = serr;
+			result = -1;
+			break;
+		}
+
+		alarm(idle_timeout);
+		alarm_occured = 0;
+
+		fgets(buffer, IDLE_BUFFER, ci->rx);
+		alarm(0);
+
+		if (strlen(buffer) > 4 && strncmp(buffer,"DONE",4)==0)
+			break;
+		else if (strlen(buffer) > 0) {
+			dbmail_imap_session_printf(self,"%s BAD Expecting DONE\r\n", self->tag);
+			result = -1;
+			break;
+		}
+		memset(buffer, 0, IDLE_BUFFER);
+
+		if (alarm_occured) {
+			alarm_occured = 0;
+			// send a keepalive message every 10 * IDLE_TIMEOUT seconds
+			// this will cause most clients to issue a DONE/IDLE cycle
+			if (idle_keepalive++ == 10) {
+				dbmail_imap_session_printf(self, "* OK\r\n");
+				idle_keepalive = 0;
+			}
+	
+			dbmail_imap_session_mailbox_status(self,TRUE);
+			continue;
+		}
+	}
+	alarm(0);
+
+	return result;
+}
+
 int dbmail_imap_session_mailbox_close(struct ImapSession *self)
 {
 	// flush recent messages from previous select
@@ -1973,14 +2121,16 @@ int dbmail_imap_session_set_state(struct ImapSession *self, int state)
 	imap_userdata_t *ud = (imap_userdata_t *) self->ci->userData;
 	switch (state) {
 		case IMAPCS_AUTHENTICATED:
-			ud->state = state;
 			memset(&ud->mailbox, 0, sizeof(ud->mailbox));
-			break;
-			
+			// change from login_timeout to main timeout
+			self->timeout = self->ci->timeout; 
+		break;
 		default:
-			ud->state = state;
-			break;
+		break;
 	}
+	ud->state = state;
+
+	TRACE(TRACE_DEBUG,"state [%d]", ud->state);
 	return 0;
 }
 
@@ -2298,7 +2448,8 @@ char **build_args_array_ext(struct ImapSession *self, const char *originalString
 				ci_write(ci->tx, "+ OK gimme that string\r\n");
 				
 				for (cnt = 0, dataidx = 0; cnt < quotedSize; cnt++) {
-					alarm(ci->timeout);	/* dont wait forever, but reset after every fgetc */
+					/* dont wait forever, but reset after every fgetc */
+					alarm(self->timeout);
 					if (! (gotc = fgetc(ci->rx)))
 						break;
 					
@@ -2320,7 +2471,7 @@ char **build_args_array_ext(struct ImapSession *self, const char *originalString
 						dbmail_imap_session_printf(self, "%s", IMAP_TIMEOUT_MSG);
 						client_close();
 						TRACE(TRACE_ERROR, "timeout occurred in fgetc; got [%d] of [%d]; timeout [%d]", 
-								cnt, quotedSize, ci->timeout);
+								cnt, quotedSize, self->timeout);
 						return NULL;
 					}
 				}
