@@ -12,17 +12,19 @@
 /* ImapSession definition */
 struct ImapSession {
 	clientinfo_t *ci;
-	gboolean use_uid;
 	u64_t msg_idnr;  // replace this with a GList
 	GMimeStream *fstream; // gmime filter wrapper around the TX handler in clientinfo_t
+	GString *buff; // use buffered writes
+	gboolean use_uid;
 	char *tag;
 	char *command;
 	int command_type;
+	int timeout;
 	char **args;
 	u64_t args_idx;
 	fetch_items_t *fi;
-	struct DbmailMessage *message;
 	struct DbmailMailbox *mailbox;
+	struct DbmailMessage *message;
 	GTree *ids;
 	GTree *headers;
 	GTree *envelopes;
@@ -30,6 +32,7 @@ struct ImapSession {
 	GList *recent;
 	GList *ids_list;
 	gpointer cmd; // command structure
+	gboolean error; // command result
 };
 
 typedef struct {
@@ -66,9 +69,13 @@ int dbmail_imap_session_discard_to_eol(struct ImapSession *self);
  *   -1 on possibly recoverable errors
  *   -2 on serious unrecoverable errors
  */
+void dbmail_imap_session_buff_clear(struct ImapSession *self);
+void dbmail_imap_session_buff_append(struct ImapSession *self, char *message, ...);
+void dbmail_imap_session_buff_flush(struct ImapSession *self);
 int dbmail_imap_session_printf(struct ImapSession * self, char * message, ...);
 
 int dbmail_imap_session_set_state(struct ImapSession *self, int state);
+int client_is_authenticated(struct ImapSession * self);
 int check_state_and_args(struct ImapSession * self, const char * command, int minargs, int maxargs, int state);
 int dbmail_imap_session_handle_auth(struct ImapSession * self, char * username, char * password);
 int dbmail_imap_session_prompt(struct ImapSession * self, char * prompt, char * value);
@@ -77,6 +84,7 @@ int dbmail_imap_session_mailbox_check_acl(struct ImapSession * self, u64_t idnr,
 int dbmail_imap_session_mailbox_get_selectable(struct ImapSession * self, u64_t idnr);
 
 int dbmail_imap_session_mailbox_status(struct ImapSession * self, gboolean update);
+int dbmail_imap_session_idle(struct ImapSession *self);
 int dbmail_imap_session_mailbox_show_info(struct ImapSession * self);
 int dbmail_imap_session_mailbox_open(struct ImapSession * self, const char * mailbox);
 int dbmail_imap_session_mailbox_close(struct ImapSession *self);
