@@ -112,7 +112,7 @@ int sort_vacation(sieve2_context_t *s, void *my)
 	message = sieve2_getvalue_string(s, "message");
 	subject = sieve2_getvalue_string(s, "subject");
 	fromaddr = sieve2_getvalue_string(s, "fromaddr"); // From: specified by the script.
-	handle = sieve2_getvalue_string(s, "handle");
+	handle = sieve2_getvalue_string(s, "hash");
 
 	/* Default to a week, upper limit of a month.
 	 * This is our only loop prevention mechanism! The value must be
@@ -156,13 +156,46 @@ int sort_vacation(sieve2_context_t *s, void *my)
 	if (md5_handle)
 		g_free(md5_handle);
 
-	m->result->cancelkeep = 0;
 	return SIEVE2_OK;
 }
 
-int sort_notify(sieve2_context_t *s UNUSED, void *my UNUSED)
+/*
+From http://www.ietf.org/internet-drafts/draft-ietf-sieve-notify-07.txt
+
+   Usage:  notify [":from" string] [":importance" <"1" / "2" / "3">]
+                  [":options" string-list] [":message" string] <method: string>
+*/
+int sort_notify(sieve2_context_t *s, void *my)
 {
-	return SIEVE2_ERROR_UNSUPPORTED;
+	struct sort_context *m = (struct sort_context *)my;
+	const char *message, *fromaddr, *method;
+	const char *rc_to, *rc_from;
+	int importance;
+	char * const * options;
+
+	fromaddr = sieve2_getvalue_string(s, "fromaddr");
+	method = sieve2_getvalue_string(s, "method");
+	message = sieve2_getvalue_string(s, "message");
+	importance = sieve2_getvalue_int(s, "importance");
+	options = sieve2_getvalue_stringlist(s, "options");
+
+	// FIXME: should be validated as a user might try
+	// to forge an address from their script.
+	rc_from = fromaddr;
+	if (!rc_from)
+		rc_from = dbmail_message_get_header(m->message, "Delivered-To");
+	if (!rc_from)
+		rc_from = m->message->envelope_recipient->str;
+
+	rc_to = dbmail_message_get_header(m->message, "Reply-To");
+	if (!rc_to)
+		rc_to = dbmail_message_get_header(m->message, "Return-Path");
+
+//	send_notification(m->message, rc_to, rc_from, method, message);
+
+	TRACE(TRACE_INFO, "Notification from [%s] to [%s] was not sent as notify is not supported in this release.", rc_from, rc_to);
+
+	return SIEVE2_OK;
 }
 
 int sort_redirect(sieve2_context_t *s, void *my)
@@ -225,10 +258,11 @@ int sort_fileinto(sieve2_context_t *s, void *my)
 	extern const char * imap_flag_desc[];
 	char * const * flags;
 	const char * mailbox;
-	int *msgflags = NULL;
+	int msgflags[IMAP_NFLAGS];
+	int *has_msgflags = NULL;
 
 	mailbox = sieve2_getvalue_string(s, "mailbox");
-	flags = sieve2_getvalue_stringlist(s, "imapflags");
+	flags = sieve2_getvalue_stringlist(s, "flags");
 
 	/* This condition exists for the KEEP callback. */
 	if (! mailbox) {
@@ -238,34 +272,63 @@ int sort_fileinto(sieve2_context_t *s, void *my)
 	/* If there were any imapflags, set them. */
 	if (flags) {
 		int i, j;
-		msgflags = g_new0(int, IMAP_NFLAGS);
+		memset(msgflags, 0, IMAP_NFLAGS * sizeof(int));
 
-		for (i = 0; flags[i]; i++) { // Loop through all script/user-specified flags.
-			for (j = 0; imap_flag_desc[j]; j++) { // Find the ones we support.
-				if (g_strcasestr(imap_flag_desc[j], flags[i])) {
-					msgflags[i] = 1;
+		// Loop through all script/user-specified flags.
+		for (i = 0; flags[i]; i++) {
+			int supported = 0;
+			// Find the ones we support.
+			for (j = 0; imap_flag_desc[j] && j < IMAP_NFLAGS; j++) {
+				// Point to the first character after the last '\'
+				char *flag = strrchr(flags[i], '\\');
+				if (g_strcasestr(imap_flag_desc[j], flag ? ++flag : flags[i])) {
+					supported = 1;
+					msgflags[j] = 1;
+					// Only pass msgflags if we found something.
+					has_msgflags = msgflags;
 				}
 			}
+			if (supported) {
+				TRACE(TRACE_DEBUG, "Adding flag [%s]", flags[i]);
+			} else {
+				TRACE(TRACE_DEBUG, "Unsupported flag [%s]", flags[i]);
+			}
 		}
 	}
 
-	TRACE(TRACE_INFO, "Action is FILEINTO: mailbox is [%s]", mailbox);
+	if (has_msgflags) {
+		// TODO: Don't do all this at lower trace levels
+		int i;
+		char text_msgflags[IMAP_NFLAGS * 10];
+		memset(text_msgflags, 0, IMAP_NFLAGS * 10);
+		extern const char * imap_flag_desc_escaped[];
+		for (i = 0; imap_flag_desc_escaped[i] && i < IMAP_NFLAGS; i++) {
+			if (msgflags[i]) {
+				g_strlcat(text_msgflags, imap_flag_desc_escaped[i], IMAP_NFLAGS * 10);
+				g_strlcat(text_msgflags, " ", IMAP_NFLAGS * 10);
+			}
+		}
+		TRACE(TRACE_INFO, "Action is FILEINTO: mailbox is [%s] flags are [%s]", mailbox, text_msgflags);
+	} else {
+		TRACE(TRACE_INFO, "Action is FILEINTO: mailbox is [%s] no flags", mailbox);
+	}
 
 	/* Don't cancel the keep if there's a problem storing the message. */
 	if (sort_deliver_to_mailbox(m->message, m->user_idnr,
-			mailbox, BOX_SORTING, msgflags) != DSN_CLASS_OK) {
+			mailbox, BOX_SORTING, has_msgflags) != DSN_CLASS_OK) {
 		TRACE(TRACE_ERROR, "Could not file message into mailbox; not cancelling keep.");
 		m->result->cancelkeep = 0;
 	} else {
 		m->result->cancelkeep = 1;
 	}
 
-	if (msgflags)
-		g_free(msgflags);
-
 	return SIEVE2_OK;
 }
 
+/* This should only happen if the user has uploaded an invalid script.
+ * Possible causes are a homebrew script uploader that does not do proper
+ * validation, libSieve's removal of a deprecated Sieve language feature,
+ * or perhaps some bugginess elsewhere. */
 int sort_errparse(sieve2_context_t *s, void *my)
 {
 	struct sort_context *m = (struct sort_context *)my;
@@ -303,6 +366,11 @@ int sort_errexec(sieve2_context_t *s, void *my)
 
 	TRACE(TRACE_INFO, "Error is EXEC: Message is [%s]", message);
 
+	/* This turns out to be incredibly annoying, as libSieve
+	 * throws execution errors on malformed addresses coming
+	 * from the wild. As you might guess, that happens with
+	 * greater than trivial frequency.
+
 	g_string_append_printf(m->result->errormsg, "Execution error: %s", message);
 
 	if (m->message) {
@@ -315,6 +383,7 @@ int sort_errexec(sieve2_context_t *s, void *my)
 		send_alert(m->user_idnr, "Sieve script run error", alertbody);
 		g_free(alertbody);
 	}
+	*/
 
 	m->result->error_runtime = 1;
 	return SIEVE2_OK;
@@ -390,10 +459,13 @@ int sort_getenvelope(sieve2_context_t *s, void *my)
 {
 	struct sort_context *m = (struct sort_context *)my;
 
-	sieve2_setvalue_string(s, "to",
-		m->message->envelope_recipient->str);
-	sieve2_setvalue_string(s, "from",
-		(char *)dbmail_message_get_header(m->message, "Return-Path"));
+	char *to = m->message->envelope_recipient->str;
+	char *from  = (char *)dbmail_message_get_header(m->message, "Return-Path");
+
+	TRACE(TRACE_INFO, "Getting envelope, to [%s] from [%s]", to, from);
+
+	sieve2_setvalue_string(s, "to", to);
+	sieve2_setvalue_string(s, "from", from);
 
 	return SIEVE2_OK;
 }
@@ -588,7 +660,7 @@ static int sort_startup(sieve2_context_t **s2c,
 		}
 	}
 	if (sieve_config.notify) {
-		TRACE(TRACE_DEBUG, "Sieve notify enabled.");
+		TRACE(TRACE_ERROR, "Sieve notify is not supported in this release.");
 		res = sieve2_callbacks(sieve2_context, notify_callbacks);
 		if (res != SIEVE2_OK) {
 			TRACE(TRACE_ERROR, "Error [%d] when calling sieve2_callbacks: [%s]",
@@ -643,7 +715,7 @@ const char * sort_listextensions(void)
 		sieve2_callbacks(sieve2_context, vacation_callbacks);
 	}
 	if (sieve_config.notify) {
-		TRACE(TRACE_DEBUG, "Sieve notify enabled.");
+		TRACE(TRACE_ERROR, "Sieve notify is not supported in this release.");
 		sieve2_callbacks(sieve2_context, notify_callbacks);
 	}
 	if (sieve_config.debug) {
@@ -770,12 +842,13 @@ sort_result_t *sort_process(u64_t user_idnr, struct DbmailMessage *message)
 			res, sieve2_errstr(res));
 		exitnull = 1;
 	}
+
+	/* At this point the callbacks are called from within libSieve. */
+
 	if (! sort_context->result->cancelkeep) {
 		TRACE(TRACE_INFO, "No actions taken; message must be kept.");
 	}
 
-	/* At this point the callbacks are called from within libSieve. */
-
 freesieve:
 	if (sort_context->s_buf)
 		g_free(sort_context->s_buf);
@@ -797,10 +870,8 @@ freesieve:
 void sort_free_result(sort_result_t *result)
 {
 	if (result == NULL) return;
-	if (result->errormsg != NULL) 
-		g_string_free(result->errormsg, TRUE);
-	if (result->rejectmsg != NULL) 
-		g_string_free(result->rejectmsg, TRUE);
+	if (result->errormsg) g_string_free(result->errormsg, TRUE);
+	if (result->rejectmsg) g_string_free(result->rejectmsg, TRUE);
 	g_free(result);
 }
 
@@ -825,18 +896,21 @@ int sort_get_reject(sort_result_t *result)
 const char *sort_get_rejectmsg(sort_result_t *result)
 {
 	if (result == NULL) return NULL;
+	if (result->rejectmsg == NULL) return NULL;
 	return result->rejectmsg->str;
 }
 
 int sort_get_error(sort_result_t *result)
 {
 	if (result == NULL) return 0;
+	if (result->errormsg == NULL) return 0;
 	return result->errormsg->len;
 }
 
 const char * sort_get_errormsg(sort_result_t *result)
 {
 	if (result == NULL) return NULL;
+	if (result->errormsg == NULL) return NULL;
 	return result->errormsg->str;
 }
 
