@@ -19,7 +19,7 @@
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/* $Id: pop3d.c 2259 2006-09-08 19:48:57Z aaron $
+/* 
 *
 * pop3d.c
 *
@@ -30,9 +30,6 @@
 #define THIS_MODULE "pop3d"
 #define PNAME "dbmail/pop3d"
 
-/* server timeout error */
-#define POP_TIMEOUT_MSG "-ERR I'm leaving, you're tooo slow\r\n"
-
 /* also used in pop3.c */
 int pop_before_smtp = 0;
 
@@ -44,6 +41,9 @@ int main(int argc, char *argv[])
 	g_mime_init(0);
 	openlog(PNAME, LOG_PID, LOG_MAIL);
 
+        //fixes valgrind Conditional jump or move depends on uninitialised value(s)
+        memset(&config, 0, sizeof(serverConfig_t));
+
 	result = serverparent_getopt(&config, "POP", argc, argv);
 	if (result == -1)
 		goto shutdown;
@@ -55,7 +55,6 @@ int main(int argc, char *argv[])
 	}
 
 	config.ClientHandler = pop3_handle_connection;
-	config.timeoutMsg = POP_TIMEOUT_MSG;
 	pop_before_smtp = config.service_before_smtp;
 
 	result = serverparent_mainloop(&config, "POP", "dbmail-pop3d");
