@@ -91,16 +91,6 @@ dsn_class_t sort_and_deliver(struct DbmailMessage *message,
 		}
 	}
 
-	/* Sieve actions:
-	 * (m = must implement, s = should implement, e = extension)
-	 * m Keep - implicit default action.
-	 * m Discard - requires us to skip the default action.
-	 * m Redirect - add to the forwarding list.
-	 * s Fileinto - change the destination mailbox.
-	 * s Reject - nope, sorry. we killed bounce().
-	 * e Vacation - share with the auto reply code.
-	 */
-
 	if (cancelkeep) {
 		// The implicit keep has been cancelled.
 		// This may necessarily imply that the message
@@ -131,6 +121,7 @@ dsn_class_t sort_deliver_to_mailbox(struct DbmailMessage *message,
 		int *msgflags)
 {
 	u64_t mboxidnr, newmsgidnr;
+	field_t val;
 	size_t msgsize = (u64_t)dbmail_message_get_size(message, FALSE);
 
 	TRACE(TRACE_INFO,"useridnr [%llu] mailbox [%s]", useridnr, mailbox);
@@ -150,7 +141,7 @@ dsn_class_t sort_deliver_to_mailbox(struct DbmailMessage *message,
 		TRACE(TRACE_DEBUG, "Checking if we have the right to post incoming messages");
         
 		mailbox_t mbox;
-		memset(&mbox, '\0', sizeof(mbox));
+		memset(&mbox, 0, sizeof(mbox));
 		mbox.uid = mboxidnr;
 		
 		switch (acl_has_right(&mbox, useridnr, ACL_RIGHT_POST)) {
@@ -180,6 +171,16 @@ dsn_class_t sort_deliver_to_mailbox(struct DbmailMessage *message,
 		}
 	}
 
+	// if the mailbox already holds this message we're done
+	GETCONFIGVALUE("suppress_duplicates", "DELIVERY", val);
+	if (strcasecmp(val,"yes")==0) {
+		const char *messageid = dbmail_message_get_header(message, "message-id");
+		if ( messageid && ((db_mailbox_has_message_id(mboxidnr, messageid)) > 0) ) {
+			TRACE(TRACE_MESSAGE, "suppress_duplicate: [%s]", messageid);
+			return DSN_CLASS_OK;
+		}
+	}
+
 	// Ok, we have the ACL right, time to deliver the message.
 	switch (db_copymsg(message->id, mboxidnr, useridnr, &newmsgidnr)) {
 	case -2:
