@@ -36,7 +36,11 @@
 #define MAX_IN_BUFFER 255
 
 #define GREETING(stream) \
-          ci_write(stream, "\"IMPLEMENTATION\" \"DBMail timsieved v%s\"\r\n", VERSION); \
+          GETCONFIGVALUE("banner", "SIEVE", banner); \
+	  if (strlen(banner) > 0) \
+            ci_write(stream, "\"IMPLEMENTATION\" \"%s\"\r\n", banner); \
+	  else \
+            ci_write(stream, "\"IMPLEMENTATION\" \"DBMail timsieved %s\"\r\n", DBMAIL_VERSION); \
           ci_write(stream, "\"SASL\" \"PLAIN\"\r\n"); \
           ci_write(stream, "\"SIEVE\" \"%s\"\r\n", sieve_extensions); \
           ci_write(stream, "OK\r\n")
@@ -57,6 +61,14 @@ static char myhostname[64];
 extern const char *sieve_extensions;
 extern volatile sig_atomic_t alarm_occured;
 
+int tims_reset(PopSession_t * session)
+{
+	memset(session,0,sizeof(PopSession_t));
+	session->state = STRT;
+	return 1;
+}
+
+
 int tims_handle_connection(clientinfo_t * ci)
 {
 	/*
@@ -67,10 +79,11 @@ int tims_handle_connection(clientinfo_t * ci)
 	int done = 1;		/* loop state */
 	char *buffer = NULL;	/* connection buffer */
 	int cnt;		/* counter */
+	field_t banner;
 
 	PopSession_t session;	/* current connection session */
 
-	memset(&session,0,sizeof(session));
+	tims_reset(&session);
 	
 	/* getting hostname */
 	gethostname(myhostname, 64);
@@ -90,7 +103,10 @@ int tims_handle_connection(clientinfo_t * ci)
 
 	while (done > 0) {
 		/* set the timeout counter */
-		alarm(ci->timeout);
+		if (session.state == STRT)
+			alarm(ci->login_timeout);
+		else
+			alarm(ci->timeout);
 
 		/* clear the buffer */
 		memset(buffer, 0, INCOMING_BUFFER_SIZE);
@@ -159,13 +175,6 @@ int tims_handle_connection(clientinfo_t * ci)
 }
 
 
-int tims_reset(PopSession_t * session)
-{
-	session->state = STRT;
-	return 1;
-}
-
-
 int tims_error(PopSession_t * session, void *stream,
 	       const char *formatstring, ...)
 {
@@ -200,6 +209,7 @@ int tims(clientinfo_t *ci, char *buffer, PopSession_t * session)
 	char *command, *value;
 	int cmdtype;
 	int indx = 0;
+	field_t banner;
 	
 	size_t tmplen = 0;
 	size_t tmppos = 0;
